Attribute VB_Name = "ModControl"
' Mike's Raycaster v0.04
' A demonstration of raycasting in Visual Basic

' ModControl.bas: Handles maps and the state of the scene.

' Copyright (c) 2008-2010 Michael Billington <michael.billington@gmail.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Option Explicit

Public CurMap(0 To 255, 0 To 255) As Byte
Public BlockW As Integer
Public BlockH As Integer
Public MapName As String

Public cView As CharPerspective

Public Type CharPerspective
    X As Double
    Y As Double
    Angle As Double
End Type

Function LoadMap(ByVal path As String)
    Dim MTemp() As String
    Dim i As Integer
    Dim j As Integer
    Dim symb As String '// ASCII symbol being read
    
    LoadMap_Initiate
    '//Check file validity? haha
    MTemp() = Split(LoadText(path), vbCrLf)
    If UBound(MTemp) > 256 Then Abort "LoadMap: Invalid map, too many lines (max 257): " & path
    '// MTemp(0) contains meta-data separated by ",": Title, X block size, Y block size, Starting angle
    BlockH = 255
    BlockW = 255
    MapName = "untitled"
    For i = 1 To UBound(MTemp)
        For j = 1 To Len(MTemp(i))
            symb = Mid(MTemp(i), j, 1)
            LoadMap_AssignSymbol j - 1, i - 1, symb
        Next j
    Next i
End Function

Sub LoadMap_Initiate()
    Dim X As Integer
    Dim Y As Integer
    X = 0: Y = 0
    Do While Y <= 255
        Do While X <= 255
            LoadMap_AssignSymbol X, Y, "0"
            If X = 255 Then Exit Do
            X = X + 1
        Loop
        If Y = 255 Then Exit Do
        Y = Y + 1
    Loop
End Sub

Function LoadMap_AssignSymbol(ByVal X As Integer, Y As Integer, Symbol As String)
    Dim YPos As Long
    Dim XPos As Long
    Select Case Symbol
    Case "X"
        CurMap(X, Y) = 16
        cView.Y = CLng(Y) * BlockH + (BlockH / 2)
        cView.X = CLng(X) * BlockW + (BlockW / 2)
    Case "0": CurMap(X, Y) = 0
    Case "1": CurMap(X, Y) = 1
    Case "2": CurMap(X, Y) = 2
    Case "3": CurMap(X, Y) = 3
    Case "4": CurMap(X, Y) = 4
    Case "5": CurMap(X, Y) = 5
    Case "6": CurMap(X, Y) = 6
    Case "7": CurMap(X, Y) = 7
    Case "8": CurMap(X, Y) = 8
    Case "9": CurMap(X, Y) = 9
    Case "A": CurMap(X, Y) = 10
    Case "B": CurMap(X, Y) = 11
    Case "C": CurMap(X, Y) = 12
    Case "D": CurMap(X, Y) = 13
    Case "E": CurMap(X, Y) = 14
    Case "F": CurMap(X, Y) = 15
    Case " ": CurMap(X, Y) = 16
    Case Else
        Abort "LoadMap_AssignSymbol: Unknown symbol '" & Symbol & "' at " & X & "," & Y & " in " & MapName
    End Select
End Function

Sub Abort(Optional ByVal Message As String)
    If Message = "" Then Message = "Unknown error: Aborting"
    MsgBox Message, vbCritical, "Operation aborted"
    Debug.Print Message
    Die
End Sub

Sub Die()
    End
End Sub



