Attribute VB_Name = "ModRender"
' Mike's Raycaster v0.04
' A demonstration of raycasting in Visual Basic

' ModRender.bas: Functions to render a scene in 3D.
'                    2D rendering is for debugging only.

' Copyright (c) 2008-2010 Michael Billington <michael.billington@gmail.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Option Explicit
Public Const pi = 3.14159265358979
Const Vis_width = 10 * pi / 18

Sub Render2D(Board As PictureBox)
    Board.Cls
    Dim BlockVal As Byte
    Dim BlockColour As Long
    Dim X As Long, Y As Long
    For Y = 0 To 16
        For X = 0 To 16
            BlockVal = CurMap(X, Y)
            Select Case BlockVal
            Case 0 To 15
                BlockColour = QBColor(BlockVal)
                Board.Line (X * BlockW, Y * BlockH)-((X + 1) * BlockW - 1, (Y + 1) * BlockH - 1), BlockColour, BF
            Case 16
                BlockColour = QBColor(7)
            Case Else
                DoEvents
            End Select
            Board.Line (X * BlockW, Y * BlockH)-((X + 1) * BlockW - 1, (Y + 1) * BlockH - 1), QBColor(7), B
        Next X
    Next Y
    Board.PSet (cView.X, cView.Y), vbRed
    Render_cast Board
End Sub

Sub Render3D(Board As PictureBox)
    On Error Resume Next
    Board.Cls
        Dim i As Long
    Dim angOffset As Double
    Dim max As Long
    
    '// Sum of partial ray lengths
    Dim RayX As Double
    Dim RayY As Double
    '//Co-ordinates for box which the ray starts in (origin)
    Dim StartBoxX As Long
    Dim StartBoxY As Long
    
    '//New Box number after the ray is resolved.
    Dim BoxX As Long
    Dim BoxY As Long
    '//Partial ray lengths
    Dim RayX_Restricted As Double
    Dim RayY_Restricted As Double
    '// Position of ray partial within box
    Dim X As Double
    Dim Y As Double
    Dim Collided As Boolean
    Dim count As Long
    Dim RayMagnitude As Double
    
    StartBoxX = cView.X \ BlockW
    StartBoxY = cView.Y \ BlockH
    
    Dim Magnitude As Double
    Magnitude = 300
    max = Board.ScaleWidth
    For i = 0 To max
        angOffset = normalise_angle((cView.Angle - Vis_width / 2) + (Vis_width / max) * i)
        
        RayX = cView.X
        RayY = cView.Y
        StartBoxX = cView.X \ BlockW
        StartBoxY = cView.Y \ BlockH
        BoxX = StartBoxX
        BoxY = StartBoxY
        Collided = False
        count = 0
        If CurMap(BoxX, BoxY) <> 16 Then Collided = True
        
        X = RayX Mod BlockW
        Y = RayY Mod BlockH
        Do
            If Resolve_Ray(X, Y, RayX_Restricted, RayY_Restricted, BoxX, BoxY, angOffset) Then
                If CurMap(BoxX, BoxY) <> 16 Then
                    Collided = True
                End If
                'Board.Line (RayX, RayY)-(RayX + RayX_Restricted, RayY + RayY_Restricted), IIf(Collided, vbRed, vbGreen)
            Else
                Collided = True
            End If
            
            RayX = RayX + RayX_Restricted
            RayY = RayY + RayY_Restricted
            
            If Not Collided Then
                If BoxX < StartBoxX And RayX Mod BlockW = 0 Then
                    X = BlockW
                Else
                    X = RayX Mod BlockW
                End If
                
                If BoxY < StartBoxY And RayY Mod BlockH = 0 Then
                    Y = BlockH
                Else
                    Y = RayY Mod BlockH
                End If
            End If
            
            StartBoxX = BoxX
            StartBoxY = BoxY
            count = count + 1
            If count > 15 Then Exit Do
        Loop While Not Collided
        RayMagnitude = Sqr((cView.X - RayX) ^ 2 + (cView.Y - RayY) ^ 2)
        'Board.Line (i, 0)-(i, Board.ScaleHeight - RayMagnitude), vbBlack
        PlotRay i, RayMagnitude, Board, CurMap(BoxX, BoxY)
    Next i
End Sub

Sub PlotRay(RayIndex As Long, Dist As Double, Board As PictureBox, Optional map As Byte)
    Dim Dist2 As Double
    Dim Darkness As Long
    Dim WallHeight As Long
    
    Dist2 = Abs(Dist)
    If Dist2 <> 0 Then WallHeight = Board.height / Dist2 * BlockH Else WallHeight = Board.height '(256 / Dist2) * (256 / (2 * Pi)) else
    If WallHeight > Board.height Then WallHeight = Board.height
    Darkness = (WallHeight / Board.height * 255) '256 - ((Dist2 - 77) / 73 * 256)
    Dim col As Long
    Select Case map
        Case 0
            col = RGB(Darkness, Darkness, Darkness)
        Case 1
            col = RGB(Darkness, 0, 0)
        Case 2
            col = RGB(0, Darkness, 0)
        Case 3
            col = RGB(0, 0, Darkness)
        Case 4
            col = RGB(Darkness, 0, Darkness)
        Case 5
            col = RGB(Darkness, Darkness, 0)
        Case 6
            col = RGB(0, Darkness, Darkness)
        Case 7
            col = RGB(Darkness / 2, Darkness / 2, Darkness / 2)
        Case Else
            col = RGB(Darkness, Darkness, Darkness)
    End Select
    Board.Line (RayIndex, (Board.height - WallHeight) / 2)-(RayIndex, (Board.height - WallHeight) / 2 + WallHeight), col
End Sub

Sub Render_cast(ByVal Board As PictureBox)
    Dim i As Long
    Dim angOffset As Double
    Dim max As Long
    
    '// Sum of partial ray lengths
    Dim RayX As Double
    Dim RayY As Double
    '//Co-ordinates for box which the ray starts in (origin)
    Dim StartBoxX As Long
    Dim StartBoxY As Long
    
    '//New Box number after the ray is resolved.
    Dim BoxX As Long
    Dim BoxY As Long
    '//Partial ray lengths
    Dim RayX_Restricted As Double
    Dim RayY_Restricted As Double
    '// Position of ray partial within box
    Dim X As Double
    Dim Y As Double
    Dim Collided As Boolean
    Dim count As Long
    Dim raylen As Long
    
    StartBoxX = cView.X \ BlockW
    StartBoxY = cView.Y \ BlockH
    Board.Line (StartBoxX * BlockW, StartBoxY * BlockH)-((StartBoxX + 1) * BlockW - 1, (StartBoxY + 1) * BlockH - 1), vbBlue, B
    
    Dim Magnitude As Double
    Dim col As Long
    Magnitude = 300
    max = 1 'Board.ScaleWidth
    For i = 0 To max
        angOffset = normalise_angle((cView.Angle - Vis_width / 2) + (Vis_width / max) * i)
        
        RayX = cView.X
        RayY = cView.Y
        StartBoxX = cView.X \ BlockW
        StartBoxY = cView.Y \ BlockH
        BoxX = StartBoxX
        BoxY = StartBoxY
        Collided = False
        count = 0
        If CurMap(BoxX, BoxY) <> 16 Then Collided = True
        
        X = RayX Mod BlockW
        Y = RayY Mod BlockH
        Do
            If Resolve_Ray(X, Y, RayX_Restricted, RayY_Restricted, BoxX, BoxY, angOffset, col) Then
                If CurMap(BoxX, BoxY) <> 16 Then Collided = True
            Else
                Collided = True
            End If
            
            RayX = RayX + RayX_Restricted
            RayY = RayY + RayY_Restricted
            
            If Not Collided Then
                If BoxX < StartBoxX And RayX Mod BlockW = 0 Then
                    X = BlockW
                Else
                    X = RayX Mod BlockW
                End If
                
                If BoxY < StartBoxY And RayY Mod BlockH = 0 Then
                    Y = BlockH
                Else
                    Y = RayY Mod BlockH
                End If
            End If
            
            StartBoxX = BoxX
            StartBoxY = BoxY
            count = count + 1
            If count > 15 Then Exit Do
        Loop While Not Collided
        raylen = Sqr((cView.X - RayX) ^ 2 + (cView.Y - RayY) ^ 2)
        Board.Line (cView.X, cView.Y)-(RayX, RayY), col 'RGB(raylen, raylen, raylen)
    Next i
    Board.CurrentX = 0: Board.CurrentY = 0: Board.Print X & " " & Y
End Sub
Function Resolve_Ray(ByVal X As Double, ByVal Y As Double, ByRef width As Double, ByRef height As Double, ByRef BoxX As Long, ByRef BoxY As Long, ByVal Angle As Double, Optional col As Long) As Boolean
    Resolve_Ray = True
    Dim Opp As Double
    Dim Adj As Double
        
    '// Test for Vertical rays
    If Cos(Angle) = 0 Then '// Vertical ray
        width = 0
        If Angle <= pi Then '// bottom
            height = BlockH - Y
            BoxY = BoxY + 1
            col = vbCyan
        Else
            height = Y '// top
            BoxY = BoxY - 1
            col = vbMagenta
        End If
        Exit Function
    End If
    
    '// Rays that hit top
    Adj = -Y
    Opp = Adj * Tan(pi / 2 - Angle)
    If X + Opp >= 0 And X + Opp <= BlockW And (Angle >= pi And Angle <= 2 * pi) Then
            height = Adj
            width = Opp
            BoxY = BoxY - 1
            col = vbGreen
            Exit Function
    End If
    
    '// Rays that hit bottom
    Adj = BlockH - Y
    Opp = Adj * Tan(pi / 2 - Angle)
    If X + Opp >= 0 And X + Opp <= BlockW And (Angle <= pi And Angle >= 0) Then
            height = Adj
            width = Opp
            BoxY = BoxY + 1
            col = vbRed
            Exit Function
    End If
    
    '// Rays that hit right wall
    Adj = BlockW - X
    Opp = Adj * Tan(Angle)
    If Y + Opp >= 0 And Y + Opp <= BlockH And (Angle <= pi / 2 Or Angle >= 3 * pi / 2) Then '// Right wall
            height = Opp
            width = Adj
            BoxX = BoxX + 1
            col = vbBlue
            Exit Function
    End If
    
    '// Left wall..
    Adj = -X
    Opp = Adj * Tan(Angle)
    If Y + Opp >= 0 And Y + Opp <= BlockH And Not (Angle <= pi / 2 Or Angle >= 3 * pi / 2) Then '// Left wall
        height = Opp
        width = Adj
        BoxX = BoxX - 1
        col = vbYellow
        Exit Function
    End If
    col = vbBlack
    Resolve_Ray = False
    Debug.Print "Could not meet ray condition " & Angle & " " & X & "," & Y
End Function

Function normalise_angle(ByVal Angle As Double) As Double
    While Angle < 0
        Angle = Angle + 2 * pi
    Wend
    While Angle > 2 * pi
        Angle = Angle - 2 * pi
    Wend
    normalise_angle = Angle
End Function

